#!/usr/bin/perl
# J. C. Lansing 06/30/04
# Script to generate input files for JCL's GLOVE program
#
# Updated 09/21/04 to be compatible with version 0.8.9
# 
# Required input:
# (1) fit type (fast, FULL, ALL, or LINEAR)
# (3) A list of Tc (in seconds), fields (in Tesla), temperatures and peakheight files, one file per field/Tc/Temp pair
#
# Sample input file format (standard NMRView rate analysis output format)
#
# 0.0 0.05 0.10 0.15 0.20 0.05 0.0
# 0 1 hn 8.1356 7.7877 7.4455 7.0641 6.8013 7.8266 8.2075
# 1 2 hn 4.0093 3.8122 3.7623 3.5420 3.3990 3.8294 3.9872
#
# First line contains delay info, for all others:
# Col	Contents
# 0		peak number
# 1		residue number
# 2		atom type
# 3+	Intensity at given time point

sub help {
	print STDERR "Script to generate GLOVE input files for R1 or R2 data.\n";
	print STDERR "Options:\n";
	print STDERR "-t TYPE\t\t\t\tSpecify fit type (CONST, KD, or EXP3)\n";
	print STDERR "-f FILE\t\tSpecify a data file\n";
	print STDERR "-F FILE Err\tSpecify a data file with the given constant error\n";
	print STDERR "-x XMIN\t\t\t\tSet the plot x minimum\n";
	print STDERR "-X XMAX\t\t\t\tSet the plot x maximum\n";
	print STDERR "-y YMIN\t\t\t\tSet the plot y minimum\n";
	print STDERR "-Y YMAX\t\t\t\tSet the plot y maximum\n";
	print STDERR "-autox\t\t\t\tSet the plot x range automatically\n";
	print STDERR "-autoy\t\t\t\tSet the plot y range automatically\n";
	print STDERR "-dx val\t\t\t\tSet the x tick spacing\n";
	print STDERR "-dy val\t\t\t\tSet the y tick spacing\n";
	print STDERR "-min PAR VAL\t\t\tSet the minimum value\n";
	print STDERR "-max PAR VAL\t\t\tSet the maximum value\n";
	print STDERR "-sz  PAR VAL\t\t\tSet the number of steps\n";
	print STDERR "-jack\t\t\t\tSelects jackknife simulations\n";
	print STDERR "-mc N\t\t\t\tSelects Monte Carlo (N simulations)\n";
	print STDERR "-nosim\t\t\t\tSelects no simulations\n";
	print STDERR "-cutoff N\t\tTrims data points smaller than N * noise\n";
	print STDERR "Options -t and -f/F are mandatory, all others are optional.\n";
	exit($_[0]);
}

if ($#ARGV < 0) { help(0); }

# Some plotting defaults
#$ymin = 0;
#$ymax = 200;

$scale=1e5;
$err=0.05;#10% of intensity

# Some parameter defaults
$min{A} = 0.1;
$max{A} = 3;
$sz{A}  = 200;
$min{B}  = -10;
$max{B}  = -0.1;
$sz{B}   = 200;
$min{C}   = -0.3;
$max{C}   = 0.3;
$sz{C}    = 21;
$cutoff   = -1e9;
$calc     = 1;

# Process commandline
$nfile = 0;
for ($i = 0; $i <= $#ARGV; $i++) {
	if ($ARGV[$i] =~ /^-h$/) {
		help(0);
	} elsif ($ARGV[$i] =~ /^-t$/) {
		$i++;
		if ($i > $#ARGV) { die "ERROR: Fit type unspecified\n"; }
		$fit_type = $ARGV[$i];
		$fit_type =~ tr/A-Z/a-z/;
	} elsif ($ARGV[$i] =~ /^-f$/) {
		# Specify an input file
		if ($i + 1 > $#ARGV) { 
			die "ERROR: Insufficient arguments to '-f'\n"; }
		$Fname[$nfile] = $ARGV[$i+1];
		$nfile++;
		$i += 1;
	} elsif ($ARGV[$i] =~ /^-F$/) {
		# Specify an input file with a constant error
		if ($i + 2 > $#ARGV) { 
			die "ERROR: Insufficient arguments to '-F'\n"; }
		$Fname[$nfile] = $ARGV[$i+1];
		$Err[$nfile]   = $ARGV[$i+5];
		$nfile++;
		$i += 2;
	} elsif ($ARGV[$i] =~ /^-y$/) {
		$i++;
		if ($i > $#ARGV) { die "ERROR: Minimum y value unspecified\n";}
		$ymin = $ARGV[$i];
	} elsif ($ARGV[$i] =~ /^-Y$/) {
		$i++;
		if ($i > $#ARGV) { die "ERROR: Maximum y value unspecified\n";}
		$ymax = $ARGV[$i];
	} elsif ($ARGV[$i] =~ /^-x$/) {
		$i++;
		if ($i > $#ARGV) { die "ERROR: Minimum x value unspecified\n";}
		$xmin = $ARGV[$i];
	} elsif ($ARGV[$i] =~ /^-X$/) {
		$i++;
		if ($i > $#ARGV) { die "ERROR: Maximum x value unspecified\n";}
		$xmax = $ARGV[$i];
	} elsif ($ARGV[$i] =~ /^-dx$/) {
		$i++;
		if ($i > $#ARGV) { die "ERROR: X tick spacing value unspecified\n";}
		$dx = $ARGV[$i];
	} elsif ($ARGV[$i] =~ /^-dy$/) {
		$i++;
		if ($i > $#ARGV) { die "ERROR: Y tick spacing value unspecified\n";}
		$dy = $ARGV[$i];
	} elsif ($ARGV[$i] =~ /^-min$/) {
		$i++;
		if ($i+1 > $#ARGV) { die "ERROR: Parameter name and value required for -min\n";}
		$nm = $ARGV[$i++];
		$min{$nm} = $ARGV[$i];
	} elsif ($ARGV[$i] =~ /^-max$/) {
		$i++;
		if ($i+1 > $#ARGV) { die "ERROR: Parameter name and value required for -max\n";}
		$nm = $ARGV[$i++];
		$max{$nm} = $ARGV[$i];
	} elsif ($ARGV[$i] =~ /^-sz$/) {
		$i++;
		if ($i+1 > $#ARGV) { die "ERROR: Parameter name and value required for -sz\n";}
		$nm = $ARGV[$i++];
		$sz{$nm} = $ARGV[$i];
	} elsif ($ARGV[$i] =~ /^-jack$/) {
		$nmonte = 0;
		$jack = 1;
	} elsif ($ARGV[$i] =~ /^-mc$/) {
		$i++;
		if ($i > $#ARGV) { die "ERROR: Number of simulations required for -mc\n";}
		$nmonte = $ARGV[$i];
		$jack = 0;
	} elsif ($ARGV[$i] =~ /^-nosim$/) {
		$nmonte = 0;
		$jack = 0;
	} elsif ($ARGV[$i] =~ /^-autox$/) {
		undef($xmin);
		undef($xmax);
	} elsif ($ARGV[$i] =~ /^-autoy$/) {
		undef($ymin);
		undef($ymax);
	} elsif ($ARGV[$i] =~ /^-cutoff$/) {
		$i++;
		if ($i > $#ARGV) { die "ERROR: Ratio required for -cutoff\n";}
		$cutoff = $ARGV[$i];
	} else { die "ERROR: parameter \'$ARGV[$i]\' not understood\n"; }
}


# Check the input parameters
if ($nfile < 1) 
	{ die "ERROR: No input files specified\n"; }
if (!defined($fit_type)) 
	{ die "ERROR: No fit type specified\n"; }
if (defined($ymax) && $ymax <= $ymin)
	{ die "ERROR: Ymax must be greater than Ymin\n"; }
if (defined($xmax) && $xmax <= $xmin)
	{ die "ERROR: Xmax must be greater than Xmin\n"; }


# Process each input file in turn
for ($i = 0; $i < $nfile; $i++) {
	# Initialize values
	undef(%I);
	undef(@delay);
	undef(@dcol);
	undef(%frq);
	undef(@list);
	undef($noise_sum);		# Sum of noise intensities
	undef($noise_sumsq);	# Sum of squares of noise intensities
	undef($noise_n);		# Number of measurements of noise intensities
	$noisecount = 0;
	# Read the file
	open(INPUT,$Fname[$i]) || die "ERROR: Unable to open file \"$Fname[$i]\".\n";
	# Get the delays and find the reference columns
	do {
		$line = <INPUT>;
		chomp($line);
	} while ($line =~ /^#/);
	@d = split(/\s+/,$line);
	$ncol = $#d;
	$j = 0;
	for ($k = 0; $k <= $#d; $k++) {
		$delay[$j] = $d[$k];	# Store the delay value
		$dcol[$j] = $k;		# Store the column number for the delay value
		$frq{$d[$k]}++;		# Count how many times that delay shows up
		$j++;				# Increment the counter
	}
	# Process data lines
	while ($line = <INPUT>) {
		# Digest the line
		chomp($line);
		@col = split(/\s+/,$line);
		if ($#col >= $ncol) {
			$pk = $col[0];
			$res = $col[1];
			$atom = $col[2];	
			$label = "$res-$atom";
			if ($atom =~ /^noise/i) {
				# Use noise peaks to determine a baseline value
				for ($j = 0; $j <= $#delay; $j++) {
					$noise_sum += $col[$dcol[$j]+3];
					$noise_sumsq += $col[$dcol[$j]+3] * $col[$dcol[$j]+3];
					$noise_n++;
				}
			} else {
				push(@list,$label);
				# Store the intensities
				undef(%sum);
				undef(%sumsq);
				for ($j = 0; $j <= $#delay; $j++) {
					$I{$label}[$j]   = $col[$dcol[$j]+4];
					$sum{$delay[$j]} += $I{$label}[$j];
					$sumsq{$delay[$j]} += $I{$label}[$j]*$I{$label}[$j];
				}
				# Process duplicated points
				if (defined($Err[$i])) {
					$dI{$label} = $Err[$i];
				} else {
					$var = $nvar = 0;
					foreach $k (keys(%sum)) {
						if ($frq{$k} > 1) {
							$var += $sumsq{$k} - $sum{$k}*$sum{$k}/$frq{$k};
							$nvar += $frq{$k};
						}
					}
					if ($nvar < 2) { $dI{$label} = 0.0; } 
					else { $dI{$label} = sqrt($var/($nvar - 1)); }
				}	# End if defined(const_err)
			}	# End if noise
		}	# End if $#d >= $ncol
	}	# End while <INPUT>
	close(INPUT);
	
	# Calculate standard deviation of the noise
		if ($noise_n > 1) {
			$noise_stdev = sqrt(($noise_sumsq - $noise_sum * $noise_sum / $noise_n) / ($noise_n - 1));
		} else { $noise_stdev = 0; }
	
	# Store the data
	foreach $label (@list) {
		# Combine errors calculated from repeat points with that of the noise points
		$stdev_set = $noise_stdev * $noise_stdev + $dI{$label} * $dI{$label};
		for ($j = 0; $j <= $#delay; $j++) {
			#if ($I{$label}[$j] > $cutoff * $stdev_set) {
			if ($I{$label}[$j] > -1e9) {
				if (defined(%Int) && defined(@Int{$label})) {
					$q = $#{@Int{$label}} +1;
				} else { $q = 0; }
				$Int{$label}[$q]    = $I{$label}[$j];
				$dInt{$label}[$q]   = $stdev_set;
			#	$time{$label}[$q]   = $delay[$j] * 1000.0;
				$time{$label}[$q]   = $delay[$j] ;
			} else {
				print STDERR "WARNING: Omitting data point SET = $label Field = ";
				print STDERR "$Field[$i] Delay = $delay[$j]\n";
				print STDERR "\tIntensity less than uncertainty * $cutoff\n";
			}
		}
	}
}


# Output global stuff
if ($fit_type =~ /^const$/i) {
	print "FUNCTION CONSTANT\n";
} elsif ($fit_type =~ /^kd$/i) {
	print "FUNCTION KD\n";
} elsif ($fit_type =~ /^exp3$/i) {
	print "FUNCTION EXP3\n";
} else { die "ERROR: Fit type \"$fit_type\" is unknown.\n"; }

# Define fit algorithm
print "XDIM 1\n";
print "METHOD GRID\n";
print "METHOD MARQUARDT\n";
print "METHOD MCMIN 300 100 0.05\n";
#print "GPAR B 1 3000 100\n";

# Set simulation type
if ($jack) { print "JACK\n"; } 
elsif ($nmonte > 0) { print "MONTE $nmonte\n"; }

# Set XMGR options
print "\@ XAXIS LABEL \" Gradient \"\n";
print "\@ YAXIS LABEL \" Intensity\"\n";
print "\@ XAXIS TICKLABEL FORMAT DECIMAL\n";
print "\@ YAXIS TICKLABEL FORMAT DECIMAL\n";
print "\@ XAXIS TICKLABEL CHAR SIZE 0.8\n";
print "\@ YAXIS TICKLABEL CHAR SIZE 0.8\n";
print "\@ XAXIS TICKLABEL PREC 2\n";
print "\@ YAXIS TICKLABEL PREC 2\n";
if (!defined($xmax) && !defined($xmin)) {
	# Search all data points for max and min x values
	undef($xmin);
	foreach $r (sort(keys(%Int))) {
		for ($i = 0; $i <= $#{@Int{$r}}; $i++) {
			if ($time{$r}[$i] < $xmin) { $xmin = $time{$r}[$i]; }
			if ($time{$r}[$i] > $xmax) { $xmax = $time{$r}[$i]; }
		}
	}
}
if (!defined($dx)) { $dx = ($xmax - $xmin) / 5; }
printf "\@ WORLD XMIN %5.3f\n", $xmin;
printf "\@ WORLD XMAX %5.3f\n", $xmax;
printf "\@ XAXIS TICK MAJOR %5.3f\n", $dx;
$dx /= 2;
printf "\@ XAXIS TICK MINOR %5.3f\n", $dx;

if (defined($ymax) || defined($ymin)) {
	if (!defined($dy)) { $dy = ($ymax - $ymin) / 10; }
	printf "\@ WORLD YMIN %5.3f\n", $ymin;
	printf "\@ WORLD YMAX %5.3f\n", $ymax;
	printf "\@ YAXIS TICK MAJOR %5.3f\n", $dy;
	$dy /= 2;
	printf "\@ YAXIS TICK MINOR %5.3f\n", $dy;
} else {
	# Allow XMGR to autoscale y for each graph
}
print "\@ with g0\n";
print "\@ g0 on\n";
print "\@ g0 label off\n";
print "\@ g0 hidden false\n";

# Define traces to include in each plot
print "PLOT Rate 0 0\n";

# Process each residue
foreach $r (sort(keys(%Int))) {
	print "TITLE $r\n";
	# Select parameters appropriate for this equation
	if ($fit_type =~ /^const$/) {
		printf "PAR C %5.5f\n", $min{C};
	} elsif ($fit_type =~ /^kd$/) {
		printf "PAR A %0.0f %0.0f %0.0f\n", $min{A},$max{A},$sz{A};
		#printf "PAR B %0.0f %0.0f %0.0f\n", $min{B},$max{B},$sz{B};
		printf "PAR B B\n";
	} elsif ($fit_type =~ /^exp3$/) {
		printf "PAR A %5.5f %5.5f %5.5f\n", $min{A},$max{A},$sz{A};
		printf "PAR B %5.5f %5.5f %5.5f\n", $min{B},$max{B},$sz{B};
		printf "PAR C %5.5f\n", $min{C};
	}	
	# Print the data
	for ($i = 0; $i <= $#{@Int{$r}}; $i++) {
#		printf "DATA\t %0.0f\t %0.1f\t %0.1f\n",$time{$r}[$i],abs($scale*($Int{$r}[$i]-$Int{$r}[0])),$err;
		#printf " DATA %8.5f  %8.5f  %8.5f\n",$Int{$r}[$i]-$Int{$r}[0],$time{$r}[$i],$err;
		#printf " DATA %5.9f  %8.5f  %8.5f\n",$time{$r}[$i],$Int{$r}[$i]/$scale,($Int{$r}[$i]/$scale)*($err);
		printf " DATA %5.9f  %8.5f  %8.5f\n",$time{$r}[$i],$Int{$r}[$i],$err;
#		printf " DATA %5.9f  %8.5f  %8.5f",$i, $#{@Int{$r}}, $#delay;
#		printf " DATA %5.9f  %8.5f  %8.5f\n",$time{$r}[$i],$Int{$r}[$i],$r;
	}
}

